<?php
namespace Ep\Wp;

use Ep\Middleware\BaseUrl;
use Ep\Wp\Plugin\PluginInterface;
use Ep\Wp\Plugin\PluginManager;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Diactoros\Response\HtmlResponse;
use Zend\Diactoros\Response\RedirectResponse;

/**
 * WordPress middleware
 *
 * @package Ep\Wp
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/wordpress
 */
class WordPress implements MiddlewareInterface
{
    /** @var array */
    protected $config;
    /** @var AdapterInterface */
    protected $db;
    /** @var PluginManager */
    protected $pluginManager;
    /** @var PluginInterface[] */
    protected $plugins = [];
    /** @var bool  */
    private $init = false;

    /**
     * WordPress constructor.
     *
     * @param array $config
     * @param AdapterInterface $db
     * @param PluginManager $pluginManager
     */
    public function __construct(array $config, AdapterInterface $db, PluginManager $pluginManager)
    {
        require_once __DIR__ . '/_functions.php';
        require_once ABSPATH . 'wp-includes/pluggable.php';

        $this->config        = $config;
        $this->db            = $db;
        $this->pluginManager = $pluginManager;
    }

    public function init(string $baseUrl = '/') {
        if (!$this->init) {
            define('WP_DEBUG', $this->config['debug'] ?? false);
            define('WP_DEBUG_DISPLAY', WP_DEBUG);

            $this->plugins();

            global $wpdb;
            $wpdb = new Core\Db($this->db);

            $this->paths($baseUrl);
            $this->globals();
            $this->includes();

            $this->init = true;
        }
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $this->init($request->getAttribute(BaseUrl::BASE_URL, '/'));

        if (defined('WP_INSTALLING')) {
            switch (intval($_GET['step'] ?? '')) {
                case 0:
                    require_once ABSPATH . 'wp-admin/includes/file.php';
                    require_once ABSPATH . 'wp-admin/includes/translation-install.php';

                    wp_download_language_pack('hr');

                    $uri = $request->getUri();
                    return new RedirectResponse(
                        $uri->withPath($request->getAttribute(BaseUrl::BASE_URL) . $uri->getPath())
                            ->withQuery('language=hr&step=1')
                    );
                case 1:
                    $_POST['weblog_title'] = $this->config['name'] ?? 'EP WordPress';
                    $_POST['user_name'] = 'Elektronika Perhoc';
                    $_POST['admin_password'] = 'Elektr0nik@Perh0c';
                    $_POST['admin_password2'] = 'Elektr0nik@Perh0c';
                    $_POST['admin_email'] = $this->config['email'] ?? 'web@ep.hr';
                    $_POST['blog_public'] = 1;
                    $_GET['step'] = 2;
                    break;
            }
        }

        if (defined('DOING_AJAX')) {
            error_reporting(0);
        }

        // Ako dopuštamo WordPressu da nastavi sa loadanjem vraćamo prazan response.
        if (defined('DOING_CRON') || defined('WP_ADMIN') || defined('WP_INSTALLING') || preg_match('#-login.php$#', $request->getUri()->getPath())) {
            return new HtmlResponse('');
        }

        if (!defined('WP_USE_THEMES')) {
            define('WP_USE_THEMES', true);
        }

        wp();

        ob_start();
        require_once ABSPATH . WPINC . '/template-loader.php';
        return new HtmlResponse(ob_get_clean());
    }

    /**
     * Postavlja putanje i url-ove.
     *
     * @param ServerRequestInterface $request
     */
    private function paths(string $baseUrl)
    {
        $baseUrl = preg_replace('#/wp(/wp-admin)?/?#', '', $baseUrl);

        $home = $baseUrl;

        define('WP_HOME', rtrim($home, '/'));
        define('WP_SITEURL', WP_HOME . '/wp');

        define('WP_CONTENT_DIR', dirname(ABSPATH) . '/wp-content');
        define('WP_CONTENT_URL', WP_HOME . '/wp-content');
    }

    /**
     * Inicijalizira globalne varijable.
     */
    private function globals()
    {
        define('WP_DEFAULT_THEME', 'flatsome');
    }

    /**
     * Inicijalizira vanjske pluginove.
     */
    private function plugins()
    {
        require_once ABSPATH . 'wp-includes/plugin.php';

        foreach ($this->config['plugins'] as $key => $value) {
            if (false === $value) {
                continue;
            }

            $name = is_numeric($key) ? $value : $key;

            if (isset($this->config['plugins'][$name]) && false === $this->config['plugins'][$name]) {
                continue;
            }

            $plugin = $this->pluginManager->get($name);
            $plugin();

            $this->plugins[$name] = $plugin;
        }
    }

    /**
     * Učitava WordPress.
     */
    private function includes()
    {
        /** @noinspection PhpUnusedLocalVariableInspection */
        $table_prefix = 'wp_';

        /** @noinspection PhpUnusedLocalVariableInspection */
        global $wc_cpdf, $extensions_uri, $nav_elements, $extensions_url,
               $revSliderVersion, $timeline_express_base;

        require ABSPATH . 'wp-settings.php';
    }
}